module cold_pool_death_m

  implicit none

contains

  subroutine cold_pool_death(z_cp0, r0, h0, eps_cp, beta, av_thvu_constant, &
       k_star, av_thvu, rate_uns, theta_vs, theta_v_uns, c_dw, av_rho, &
       return_code, wake_lifetime, wake_final_radius, wake_final_height, t, &
       yp, dydt_p, nbad)

    use cold_pool_death_condition_m, only: cold_pool_death_condition, &
         delta_death
    use derivs_m, only: derivs
    use derivs_param, only: set_module_var
    use numer_rec_95, only: rkqs, odeint_all_points

    real, intent(in):: z_cp0 ! initial height of cold pool, in m
    real, intent(in):: r0 ! initial radius of cold pool, in m
    real, intent(in):: h0 ! initial height of environmental boundary layer, in m

    real, intent(in):: eps_cp
    ! entrainment coefficient at the border of the cold pool

    real, intent(in):: beta ! entrainment at the top of the mixed layer
    logical, intent(in):: av_thvu_constant
    real, intent(in):: k_star

    real, intent(in):: av_thvu ! average on cold pool height
    ! of virtual potential temperature outside the cold pool, in K

    real, intent(in):: rate_uns
    ! mass rate in one unsaturated downdraft, in kg s-1

    real, intent(in):: theta_vs
    ! virtual potential temperature at the surface, in K

    real, intent(in):: theta_v_uns
    ! virtual potential temperature of the unsaturated downdraft, in K

    real, intent(in):: C_dw ! drag coefficient under cold pools, in m s-1
    real, intent(in):: av_rho ! average mass density in the cold pool, in kg m-3
    integer, intent(out):: return_code
    REAL, intent(out):: wake_lifetime ! in s
    REAL, intent(out):: wake_final_radius, wake_final_height ! in m
    real, allocatable, intent(out), optional:: t(:) ! (n) time, in s

    real, allocatable, intent(out), optional:: yp(:, :) ! (5, n)
    ! yp(:, i) is (/theta_v_cp, z_cp, r, av_thvu, h/) at date t(i)

    real, allocatable, intent(out), optional:: dydt_p(:, :) ! (5, n)
    ! time derivative of yp, in units of yp per s

    integer, intent(out), optional:: nbad
    ! number of bad (but retried and fixed) steps taken

    ! Local:

    integer n
    real delta_1 ! delta theta_v at t(n - 1)
    real delta_2 ! delta theta_v at t(n)
    real w ! weight of interpolation between t(n - 1) and t(n)
    real, parameter:: max_lifetime = 86400. * 30. ! in s

    real, allocatable:: t_local(:) ! (n) t
    real, allocatable:: yp_local(:, :) ! (5, n) yp

    !----------------------------------------------------------

    if (rate_uns < 0) then
       ! No cold pool
       wake_lifetime = - 4.
       wake_final_radius = - 4.
       wake_final_height = - 4.
       return_code = 0
    else if (theta_vs < av_thvu) then
       ! Cold pool temperature would not converge to environment temperature
       wake_lifetime = - 3.
       wake_final_radius = - 3.
       wake_final_height = - 3.
       return_code = 0
    else
       if (theta_v_uns >= av_thvu) then
          ! Bad vertical profile
          wake_lifetime = - 2.
          wake_final_radius = - 2.
          wake_final_height = - 2.
          return_code = 0
       else
          call set_module_var(eps_cp, beta, av_thvu_constant, k_star, &
               theta_vs, rate_uns, av_rho, theta_v_uns, c_dw)
          call odeint_all_points(derivs, rkqs, t_local, yp_local, return_code, &
               (/theta_v_uns, z_cp0, r0, av_thvu, h0/), nbad, dydt_p, &
               x1 = 0., x2 = max_lifetime, eps = 1e-5, dmin = 0., &
               stop_condition = cold_pool_death_condition)
          n = size(t_local)

          if (n == 1) then
             ! Dead-born
             wake_lifetime = - 1.
             wake_final_radius = - 1.
             wake_final_height = - 1.
          else if (cold_pool_death_condition(yp_local(:, n))) then
             ! From the last two integration points, linearly interpolate to
             ! delta theta_v = delta_death to find wake_lifetime:
             delta_1 = yp_local(1, n - 1) - yp_local(4, n - 1)
             delta_2 = yp_local(1, n) - yp_local(4, n)
             w = (delta_death - delta_1) / (delta_2 - delta_1)
             wake_lifetime = w * t_local(n) + (1. - w) * t_local(n - 1)
             wake_final_radius = w * yp_local(3, n) &
                  + (1. - w) * yp_local(3, n - 1)
             wake_final_height = w * yp_local(2, n) &
                  + (1. - w) * yp_local(2, n - 1)
          else
             ! Cold pool did not die.
             wake_lifetime = - 5.
             wake_final_radius = - 5.
             wake_final_height = - 5.
          end if

          if (present(t)) t = t_local
          if (present(yp)) yp = yp_local
       end if
    end if

  end subroutine cold_pool_death

end module cold_pool_death_m
